function g = GUIcon(options)
% This window controls the solution of the advection-diffusion PDE.
% 
% Choose a Peclet number. For diffusion of solutes in water, try 1000 
% times the Reynolds number. For diffusion of heat in water, try 10 times
% the Reynold number. (These values are approximate).
% 
% Choose a lambda number. This number controls the ratio of molecular
% diffusion to total diffusion and should be between 0 and 1. (I.e.,
% 1 corresponds to pure molecular diffusion)
% 
% Choose a boundary condition at the surface of the sphere. A Dirichlet
% condition says that the concentration (or temperature) at the surface
% is fixed and constant. A Neumann condition says that the radial
% derivative is fixed and constant. If you use a stream function which
% satisfies no-slip (Stokes or Numerical Savier-Stokes) then this
% implies that the flux per surface area of the sphere is constant.
% 
% Choose a way to normalize the solution. Either normalize the boundary
% condition. For a Dirichlet BC, this means to set C=1 at the surface of
% sphere. For a Neumann BC, this means to set the dC/dr=1 at the sphere.
% Alternatively, normalize the flux. This means that the solution is
% scaled so that the total flow away from the sphere is one.
% 
% Choose a discretization scheme. For low Peclet numbers, try a second
% order central scheme which is fast. At high Peclet numbers you probably
% need the upwind scheme to avoid unboundedness (seen as ripples in the
% solution).
% 
% Hit the ''solve'' button to start the computation.
% 
% Choose a way to plot the solution: Contour plots, surf plots, or plots
% of the field long the polar line or in the equitorial plane. For a 
% contour plot, you may have automatically chose contour values or you
% may specify themselves, e.g. [1,2,3] or 1:3. 
% 
% You can save the solution, and you can reload it again.

% DOCUMENTATION FOR THE DEVELOPER
% 
% GUIcon - constructor for the GUIcon object
%
% Syntax: g = GUIcon(options)
%
% IF no GUIcon object exists, then one is constructed when this
% function is called. In this case options (if present) ...
%
% At most one GUIcon object can live at once.
%
% If there exists a GUIcon object, then the options is interpreted
% as an action to modify this object and is passed on to GUIconAction.
%
% The GUIcon object is returned in g. 
%
persistent TheGUIcon

if nargin==0,
   options = [];
end

if isempty(TheGUIcon),
    TheGUIcon = 1; % Hack to avoid recursion
    TheGUIcon = GUIconFig;

   TheGUIcon = class(TheGUIcon,'GUIcon');
   TheGUIcon = GUIconAction(TheGUIcon,'refresh');
else
   if ~isempty(options),
      TheGUIcon = GUIconAction(TheGUIcon,options);
   end
end

if nargout ==1,
   g = TheGUIcon;
end

return

function args = GUIconFig()

blocky = 4.25;
editx = 70;
editdim = [14 1.5];
textdim = editdim;
totdim = [30 1.5];
textx = 55;

args.BCs = {'Dirichlet','Neumann'};
args.PlotStyles = {'Contour','Surf','Pole','Equator','Local Sherwood'};
args.Methods = {'2nd order central','4th order central','3-4 order upwind'};

ConAxesPos = [4 2 48 20];
Peclet = 10;

% Initialise fields; to be filled later
args.Field = [];
args.TensMat = [];
args.FlowField = [];
args.AutoCLVS = [];
args.UserCLVS = [];

h0 = figure('Units','char', ...
    'Name','Concentration field',...
	'ResizeFcn','Resize(GUIcon)',...
    'PaperPositionMode','auto',...
	'Color',[0.8 0.8 0.8], ...
	'Position',[88 18 85 27], ...
	'Tag','ConFig');
args.FigHandle = h0;

h1 = axes('Parent',h0, ...
	'Units','char', ...
	'CameraUpVector',[0 1 0], ...
	'CameraUpVectorMode','manual', ...
	'Color',[1 1 1], ...
	'Position',ConAxesPos, ...
	'Tag','GUIconAxes', ...
	'XColor',[0 0 0], ...
	'YColor',[0 0 0], ...
	'ZColor',[0 0 0]);
args.AxesHandle = h1;

%h2 = text('Parent',h1, ...
%	'Color',[0 0 0], ...
%	'HandleVisibility','off', ...
%	'HorizontalAlignment','center', ...
%	'Position',[0.4987468671679197 -0.06417112299465244 9.160254037844386], ...
%	'Tag','Axes1Text4', ...
%	'VerticalAlignment','cap');
%set(get(h2,'Parent'),'XLabel',h2);
%h2 = text('Parent',h1, ...
%	'Color',[0 0 0], ...
%	'HandleVisibility','off', ...
%	'HorizontalAlignment','center', ...
%	'Position',[-0.07268170426065163 0.497326203208556 9.160254037844386], ...
%	'Rotation',90, ...
%	'Tag','Axes1Text3', ...
%	'VerticalAlignment','baseline');
%set(get(h2,'Parent'),'YLabel',h2);
%h2 = text('Parent',h1, ...
%	'Color',[0 0 0], ...
%	'HandleVisibility','off', ...
%	'HorizontalAlignment','right', ...
%	'Position',[-0.06516290726817042 1.050802139037433 9.160254037844386], ...
%	'Tag','Axes1Text2', ...
%	'Visible','off');
%set(get(h2,'Parent'),'ZLabel',h2);
%h2 = text('Parent',h1, ...
%	'Color',[0 0 0], ...
%	'HandleVisibility','off', ...
%	'Position',[0.4987468671679197 1.018716577540107 9.160254037844386], ...
%	'Tag','Axes1Text1', ...
%	'VerticalAlignment','bottom');
%set(get(h2,'Parent'),'Title',h2);

h1 = uicontrol('Parent',h0, ...
	'Units','char ', ...
	'BackgroundColor',[1 1 1], ...
	'Callback','GUIcon(''peclet'')', ...
	'ListboxTop',0, ...
	'Position',[editx 19.5+blocky editdim], ...
	'String',Peclet, ...
	'Style','edit');
args.PecletEditHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[textx 19.5+blocky textdim], ...
	'String','Peclet', ...
	'Style','text');
args.PecletTextHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'BackgroundColor',[1 1 1], ...
	'Callback','GUIcon(''lambda'')', ...
	'ListboxTop',0, ...
	'Position',[editx 17.8+blocky editdim], ...
	'String','1', ...
	'Style','edit');
args.LambdaEditHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[textx 17.8+blocky textdim], ...
	'String','0<Lambda<1', ...
	'Style','text');
args.LambdaTextHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''both'')', ...
   'Value',1,...
	'ListboxTop',0, ...
	'Position',[textx 5+blocky totdim], ...
	'String','Show both sides', ...
	'Style','checkbox');
args.BothHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''cbar'')', ...
	'ListboxTop',0, ...
   'Position',[textx 6.5+blocky totdim], ...
	'String','Show colorbar', ...
	'Style','checkbox');
args.CbarHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''auto'')', ...
	'ListboxTop',0, ...
   'Position',[textx 3.5+blocky totdim], ...
	'String','Auto contour levels', ...
   'Style','checkbox', ...
   'Value', 1 );
args.AutoHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''style'')', ...
	'ListboxTop',0, ...
	'Position',[editx 8.5+blocky editdim], ...
	'String',args.PlotStyles, ...
	'Style','popupmenu', ...
	'Value',2);
args.StyleMenuHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[textx 8.5+blocky textdim], ...
	'String','Plot Style', ...
	'Style','text');
args.StyleTextHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''method'')', ...
	'ListboxTop',0, ...
	'Position',[textx 10.5+blocky totdim], ...
	'String',args.Methods, ...
	'Style','popupmenu', ...
	'Value',3);
args.MethodMenuHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''normal'')', ...
	'ListboxTop',0, ...
	'Position',[editx 14+blocky editdim], ...
	'String',{'BC','Flow'}, ...
	'Style','popupmenu', ...
	'Value',1);
args.NormMenuHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''bc'')', ...
	'ListboxTop',0, ...
	'Position',[editx 15.8+blocky editdim], ...
	'String',args.BCs, ...
	'Style','popupmenu', ...
	'Value',1);
args.BcMenuHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[textx 1.5+blocky totdim], ...
	'String','User C-levels', ...
	'Style','text');
args.UserCLevTextHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''user'')', ...
	'BackgroundColor',0.5*[1 1 1], ...
	'ListboxTop',0, ...
	'Position',[textx blocky totdim], ...
	'String','1:10', ...
	'Style','edit');

args.UserCLevHandle= h1;


h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''save'') ', ...
	'ListboxTop',0, ...
   'Position',[textx -2+blocky textdim], ...
	'String','Save');
args.SaveButtonHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''load'') ', ...
	'ListboxTop',0, ...
	'Position',[textx -4+blocky textdim], ...
	'String','Load');
args.LoadButtonHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','GUIcon(''solve'') ', ...
	'ListboxTop',0, ...
	'Position',[editx -2+blocky editdim], ...
	'String','Solve', ...
	'Tag','ConSolveButton');
args.SolveButtonHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'Callback','helpwin(''GUIcon'')', ...
	'ListboxTop',0, ...
	'Position',[editx -4+blocky editdim], ...
   'String','Help');
args.HelpButtonHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[textx 15.8+blocky textdim], ...
	'String','Inner BC', ...
	'Style','text');
args.BcTextHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[textx 12+blocky totdim], ...
	'String','Method', ...
	'Style','text');
args.MethodTextHandle = h1;

h1 = uicontrol('Parent',h0, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[textx 14+blocky textdim], ...
	'String','Normalize', ...
	'Style','text');
args.NormTextHandle = h1;

args.AxesPos = ConAxesPos;

return
