function g = GUIgrid(options)
% This window specifies the computational grid.  Only rectangular grids
% in the r, theta plane are supported.
% 
% The grid is specified by its maximum radius, and the number of angular
% and radial gridpoint.
% 
% In addition you specify the skewness through the Gamma parameter.
% With Gamma=0 you get a uniform grid, while Gamma close to 1 gives more
% gridpoints downstream. This is good for the advection-diffusion
% equation when the Peclet number is high (i.e. advection dominates).
% 
% Hint: If you don''t know your machines performance, then start with
% small and crude grids and gradually increase size and resolution.

% DOCUMENTATION FOR THE DEVELOPER
%
% GUIgrid - constructor for the GUI grid object
%
% Syntax: g = GUIgrid(options)
%
% IF no GUIgrid object exists, then one is constructed when this
% function is called. In this case options (if present) is passed
% on to the compgrid constructor.
%
% At most one GUIgrid object can live at once.
%
% If there exists a GUIgrid object, then the options is interpreted
% as an action to modify this object and is passed on to GUIgridAction.
%
% The GUIgrid object is returned in g. 
%
persistent TheGUIgrid

if nargin==0,
   options = [];
end

if isempty(TheGUIgrid),
   TheGUIgrid = 1;   % Hack to stop recursion
   Grid = compgrid(options);
   TheGUIgrid = GUIgridFig(Grid);
   set(TheGUIgrid.FigHandle,'UserData',Grid);

   TheGUIgrid = class(TheGUIgrid,'GUIgrid');
   
   TheGUIgrid = GUIgridAction(TheGUIgrid,'refresh');
else
   if ~isempty(options)
      TheGUIgrid = GUIgridAction(TheGUIgrid,options);
   end
end

if nargout ==1,
   g = TheGUIgrid;
end

return

function LayOut = GUIgridFig(cg)
% DO NOT EDIT THIS FIGURE WITH THE MATLAB TOOL GUIDE.

% Lay-out parameters
figpos = [1 32 58 18];
axespos = [4 2 26 13];
textdim = [14 1.5];
blockx = 32;
blocky = 0.5;
editdim = [10 1.5];
editx = 46;
ydiff = 1.5;

% Initial values in the edit-fields
rm = rmax(cg);
M = GetRadNum(cg);
N = GetAngNum(cg);
gamma = GetGamma(cg);

LayOut.FigHandle = figure('Color',[0.8 0.8 0.8], ...
	'units','char',...
	'Position',figpos, ...
   'PaperPositionMode','auto',...
	'ResizeFcn','Resize(GUIgrid)',...
	'Name','Grid editor',...
	'Tag','GridFig');
LayOut.AxesHandle = axes('Parent',LayOut.FigHandle, ...
	'CameraUpVector',[0 1 0], ...
	'CameraUpVectorMode','manual', ...
	'Color',[1 1 1], ...
        'units','char',...
	'Position',axespos, ...
	'Tag','GridAxes');
h2 = text('Parent',LayOut.AxesHandle, ...
	'Color',[0 0 0], ...
	'HandleVisibility','off', ...
	'HorizontalAlignment','center', ...
	'Tag','Axes1Text1', ...
	'VerticalAlignment','bottom');
set(get(h2,'Parent'),'Title',h2);
LayOut.MTextHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[blockx blocky+6*ydiff textdim], ...
	'String','No. of angles', ...
	'Style','text', ...
	'Tag','StaticText1');
%	'Position',[300 281.25 55 18.75], ...
LayOut.NTextHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[blockx blocky+5*ydiff textdim], ...
	'String','No. of radii', ...
	'Style','text', ...
	'Tag','StaticText1');
LayOut.RTextHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[blockx blocky+4*ydiff textdim], ...
	'String','Max. radius', ...
	'Style','text', ...
	'Tag','StaticText1');
LayOut.GTextHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[blockx blocky+3*ydiff textdim], ...
	'String','0<Gamma<1', ...
	'Style','text', ...
	'Tag','StaticText1');
LayOut.NEditHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Units','char', ...
	'BackgroundColor',[1 1 1], ...
	'Callback','GUIgrid (''theta'') ', ...
	'ListboxTop',0, ...
	'Position',[editx blocky+6*ydiff editdim], ...
	'String',num2str(N), ...
	'Style','edit', ...
	'Tag','GridThetaField');
LayOut.MEditHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Units','char', ...
	'BackgroundColor',[1 1 1], ...
	'Callback','GUIgrid (''radius'') ', ...
	'ListboxTop',0, ...
	'Position',[editx blocky+5*ydiff editdim], ...
	'String',num2str(M), ...
	'Style','edit', ...
	'Tag','GridRadiusField');
LayOut.REditHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Units','char', ...
	'BackgroundColor',[1 1 1], ...
	'Callback','GUIgrid (''Rmax'') ', ...
	'ListboxTop',0, ...
	'Position',[editx blocky+4*ydiff editdim], ...
	'String',num2str(rm), ...
	'Style','edit', ...
	'Tag','GridRmaxField');
LayOut.GEditHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Units','char', ...
	'BackgroundColor',[1 1 1], ...
	'Callback','GUIgrid (''gamma'') ', ...
	'ListboxTop',0, ...
	'Position',[editx blocky+3*ydiff editdim], ...
	'String',num2str(gamma), ...
	'Style','edit', ...
   'Tag','GridGammaField');
LayOut.BothHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Callback','GUIgrid (''symmetry'') ', ...
	'Units','char', ...
	'ListboxTop',0, ...
	'Position',[blockx blocky+ydiff editdim(1)+textdim(1) editdim(2)], ...
	'String','Show both sides', ...
   'Value',1,...
	'Style','checkbox', ...
	'Tag','GridSymmetric');
LayOut.HelpHandle = uicontrol('Parent',LayOut.FigHandle, ...
	'Units','char', ...
	'Callback','helpwin(''GUIgrid'')', ...
	'ListboxTop',0, ...
	'Position',[blockx blocky editdim], ...
   'String','Help');

%GUIgrid('new')

return
