function F=Flow(nf,ff,Pe,La,rs)
% Flow - computes the transport flow away from origo
%
% F=Flow(nf,ff,Pe,La,r)
%
% computes the solute flow of NumField away from the origin under the flowfield 
% ff at Peclet number Pe and Lambda number La (this controls the ratio of molecular
% diffusion to total diffusion). The computed flow is through a sphere centered 
% in the origin. r can be supplied, in which case the flow is computed at 
% (approximately) this radius, or omitted, in which case a mean flow 
% over all radia is returned.

if ~eq(compgrid(nf),compgrid(ff)),
   error('To compute solute flow, the NumField and flowfield must have identical compgrids.')
end

ur = UR(ff);
ut = UT(ff);

dnf = diff(nf);

R= RR(nf);
T = TT(nf);

Fadv = ur.*Val(nf);
Fdiff2 = - (La + (1-La)*R.^(4/3)).* dnf.dr;
Ftot = Fadv + Fdiff2/Pe;

% Integrand
y = 2*pi* Ftot .* R.^2 .* sin(T);

% Prepare for spline interpolation
T = T(1,:);
Tib = [0,T,pi];
Yib = [zeros(size(R,1),1),y,zeros(size(R,1),1)];
PP = spline(Tib,Yib);

Nint = 10000;
Tint = linspace(0,pi,Nint+1);
Yint = ppval(PP,Tint);

% Integrate by extended trapez over the interpolated table
F = (sum(Yint,2)-0.5*Yint(:,1)-0.5*Yint(:,end))*pi/Nint;

if nargin==5,
   R=R(:,1);
   F = interp1(R,F,rs);
else
   if length(F)>20,
      F = F(1:20);
   end
   
   F = mean(F);
end
