function plot(NF,option)
% plot - plot Numerical Field
%
% Syntax: plot(NF,option)
%
% Plots the numerical scalar field given by NF.
%
% The option can be omitted, or a structure containing one or more of
% the following fields
%
%   option.style   'contour' (default), 'surf', 'pole', or 'equator'
%   option.cbar    0 (default), or 1     (1 for colorbar, 0 for none)
%   option.size    'half' (defualt), or 'full'
%   option.bw      Produces black/white plot if field is present
%   option.CLVS    Contour levels to be used in a contour plot
%                  (if missing, they will be chosen automatically)
%   option.labels  Labels for contour lines. This is passed on as third
%                  argument to clabel (try 'help clabel' for details). 
%                  A good shot is 'manual' which lets you pick'em yourself.
%
% If in doubt about the options, try 
% >> opt = struct('style','surf','cbar',0)
% >> plot(NF,opt)
%
% See also: NumField, compgrid
%

%%%%%%%%%%%%%% Set default options whenever options are missing %%%%%%%%%%%

if nargin<2,
	option = struct('style','surf','cbar',0,'size','full');
end

if ~isfield(option,'style')
   option.style='surf';
end

if ~isfield(option,'cbar')
   option.cbar=0;
end

if ~isfield(option,'size')
   option.size='full';
end

%%%%%%%%%%%%%% If this is a 2-D plot ... %%%%%%%%%%%
if strcmpi(option.style,'pole')
   [R,T] = RT(NF);
   
   [M,N] = size(NF);
   
   [R,C] = PolarValue(NF);
   plot(R,C,'b-')
   return
end

if strcmpi(option.style,'equator')
   [R,T] = RT(NF);
   
   [M,N] = size(NF);
   
   C = interp(NF,R(:,1),pi/2*ones(M,1));
   
   plot(R(:,1),C)
   if strcmpi(option.size,'full')
      hold on 
      
      plot(-R(:,1),C)
      hold off
   end

	return
end
%%%%%%%%%%%%%% If we pass this line, then it's a 3-D plot %%%%%%%%%%%%

%%%%%%%%%%%%%% Compute x,z coordinates from computational grid %%%%%%%%%%%
X = XX(NF);
Z = ZZ(NF);
C = Val(NF);

%%%%%%%%%%%%%% If a full plot is wanted, then mirror data %%%%%%%%%%%

if strcmpi(option.size,'full')
   X = [X,-fliplr(X),X(:,1)];
   Z = [Z,fliplr(Z),Z(:,1)];
%   if NF.ZeroPole,
%      warning('Maybe we should do something about enforcing 0 at the poles ...')
%   end   
   C = [C,fliplr(C),C(:,1)];
end

%%%%%%%%%%%%%% Plot data in surf or contour plot %%%%%%%%%%%
if strcmpi(option.style,'surf'),
   warning off
   pcolor( X,Z,C )
   shading flat
   shading interp
   warning on
else
%   disp('HARD CODING MANUAL CLABELS IN NUMFIELD/PLOT')
%   option.labels = 'manual';
   
   if isfield(option,'CLVS')
      if all(size(option.CLVS)==[1 1]),
         option.CLVS=option.CLVS*[1 1];
      end
      if isfield(option,'bw')
         [c,h]=contour( X,Z,C,option.CLVS,'k');
      else
         [c,h]=contour( X,Z,C,option.CLVS);
      end
   elseif isfield(option,'bw')
      [c,h]=contour( X,Z,C,'k');
   else
      [c,h]=contour( X,Z,C);
   end   
         
   if isfield(option,'labels')
    	clabel(c,h,option.labels);   
   end
end

title(getname(NF))

axis equal
hold on 

%%%%%%%%%%%%%% Do we want a colorbar on the plot?  %%%%%%%%%%%
if option.cbar
   colorbar
end

%%%%%%%%%%%%%% Add a full or a half circle %%%%%%%%%%%

if strcmpi(option.size,'full')
   plotcircle([],'full')
else
	plotcircle([],'half')
end

hold off

zoom on 
return
