function [psi,ur,ut,vor,stopiter] = nnss_psi(nsf)
% This is the actual Numerical Navier-Stokes Solver.
%
% [psi,ur,ut,vor,stopiter] = nnss_psi(nsf)
%
% returns matrices describing streamfunction, radial and tangential 
% velocity, and vorticity.
%
% All parameters for the computation - Reynolds number,
% computational grid, and the initial guess, is kept in nsf which 
% is a NumStreamFct object.
%
% If the user interrupts the computation, the variable stopiter contains 
% the number of iterations performed.

global psi ur ut D2psi vor D D2 R T sT sTR sTR2 cotT cT gr D2modDir nit dt
global FixInnerVor VorDirAtPole PsiDirFore PsiDirAft

% Hard-coded options. Setting any of these to 1 make the iteration converge rapidly
% to a slightly wrong solution, which is useful when trying out new iteration schemes.
FixInnerVor = 0;
VorDirAtPole = 0;
PsiDirFore = 0;
PsiDirAft = 0;

% Show progress window
[dh,stophandle] = dialog;
progress = uint8(255+zeros(1,nsf.maxit,3));

% Set the global variable gr equal to the grid in nsf
gr = nsf.cg;

% Geometrics
R = RR(gr);
T = TT(gr);
sT = sin(T);
sTR = XX(gr);
sTR2 = sTR.*R;
cotT = cot(T);
cT = cos(T);

% Get differential operators
DiffOps;

% Get initial stokes velocities and stream function
if nsf.startiter,
   ur = nsf.ur;
   ut = nsf.ut;
   psi = nsf.val;
else
   psi_stokes = streamfct('stokes');
   psi = psi_stokes(R,T,nsf.Re);
   [ur,ut] = PolarVel(psi_stokes,R,T,nsf.Re);
end

% Start iterative solution of Navier-Stokes equations

% Import the number of iterations already performed 
% (if we are resuming an interupted computation)
nit = nsf.startiter;
progress(1,1:nit,1:2) = 0;

while 1
   nit = nit + 1;
   
   % Update progress window
   progress(1,nit,1:2) = 0;
   figure(dh);
   image(progress)
   set(gca,'xtick',[])
   set(gca,'ytick',[])
   drawnow;
   
   % Do the actual computational iteration
   psiold = psi;
   SolveD2psi(nsf.Re);
   drawnow;
	SolvePsi;
   drawnow;
   PsiToU;
   drawnow;
   
   % Check for user interrupts
   if get(stophandle,'Value')
      break
   end
   
   % Check for convergencec
   if (max(max(abs(ut(1,:)))) < nsf.tolerance) & (norm(psiold-psi,1) < nsf.tolerance),
      break
   end
   
   % Emergency stop:
   if nit > nsf.maxit,
      warndlg(['In the solution of the N-S equations, the number of iterations exceeded ',...
         'the maximum. At the end of the iteration, the maximum flow on the surface was ',...
         num2str(norm(ut(1,:),inf)),' and in the last iteration, the maximum change of the ',...
         'stream function was ',num2str(norm(psiold-psi,inf)),'. The flow field ',...
         'should not be trusted. To improve this situation, reduce the Reynolds number (!) ',...
         'or use a finer grid.']);
      break
   end
end

PsiToVor;

if nargout>4
   if get(stophandle,'Value')
      stopiter = nit;
   else
      stopiter = 0;
   end
end

close(dh)

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function DiffOps
global gr psi ur ut vor D D2 R T sT sTR sTR2 cotT cT D2modDir
global FixInnerVor VorDirAtPole PsiDirFore PsiDirAft

%D = FourthOrderUpwind(flowfield(ur,ut,gr),1);
D = FourthOrderCentral(gr,struct('zeropole',1));
[M,N] = size(gr);

D2 = D.dr2 + 1./R.^2*D.dth2 - cotT./R.^2 * D.dth;

% D2 operator modified for Dirichlet boundary conditions
D2modDir = D2;

% Inner and outer boundaries
D2modDir = SetUnity(D2modDir,ones(1,N),1:N);
D2modDir = SetUnity(D2modDir,M*ones(1,N),1:N);

% For and aft boundaries
if PsiDirFore,
   D2modDir = SetUnity(D2modDir,1:M,N*ones(1,M));
end

if PsiDirAft,
	D2modDir = SetUnity(D2modDir,1:M,ones(1,M));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function SolveD2psi(Re)
global gr psi ur ut D2psi vor D D2 R T sT sTR sTR2 cotT cT nit dt
global FixInnerVor VorDirAtPole PsiDirFore PsiDirAft
persistent A qB dB qu a RX

% Differential operator 
A = D2 - Re*(ur*D.dr + ut./R * D.dth - 2* (cotT .* ut + ur)./R * eye(D.dr));

[M,N] = size(R);

% Set up differential operators for Dirichlet type conditions 
% at inner and outer boundary
A = SetUnity(A,ones(1,N),1:N);
A = SetUnity(A,M*ones(1,N),1:N);

% Right hand side
B = sparse([],[],[],M,N,2*N);

% At outer boundaries (free flow) D2psi is 0 ....  B(M,:) = zeros(1,N);

if nit<2 | FixInnerVor,
   % First iteration:
   % At the inner boundary we take the Stokes solution ...
   % B(1,:) = 3/2*sT(1,:).^2;
   % .. or the Proudman-Pearson Inner solution 
   B(1,:) = 0.5*sT(1,:).^2.*(3 + Re*(9/8 - 1.5*cT(1,:)));
   qB = B(1,:); % zeros(size(B(1,:)));
else
   if nit>2,
      % estimate the parameter a
      % based on the model:	du = a dB + e
      % 								a = qa + epsilon
      % Where e has constant intensity and epsilon has intensity 
      % proportional to dB
      
      % Increment in the u_theta at the surface
      du = ut(1,:) - qu;
      
      % Innovation
      inno = (du.*sTR(1,:) - a * dB )';
      % Forgetting factor; from the variance of epsilon, i.e. dB
      dB2 = dB*dB';
      
      lambda = max(0.8,1 - inno'*inno / 100*(1 + dB2/max(0.1,RX)));
      lambda = min(0.95,lambda);
      
      % "Kalman filter update"
      % Variance
      RX = lambda * RX + dB2;
      % Estimate
      a = a + RX\ ( dB * inno );

      % Dead-beat control law
      k = -1/a;
   else
      RX = 1e-6;
      k = - 1;
      a = 1;
   end
   
   B(1,:) = qB + k*ut(1,:).*sTR(1,:);
end

% Save boundary condition for next iteration
dB = B(1,:) - qB;
qB = B(1,:);
qu = ut(1,:);

% Fore and aft
if VorDirAtPole,
	A = SetUnity(A,1:M,ones(1,M));
   A = SetUnity(A,1:M,N*ones(1,M));
   B(:,1) = zeros(M,1);
   B(:,N) = zeros(M,1);
end

D2psi = mldivide(A,B);

return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function SolvePsi
global D2modDir gr psi ur ut D2psi vor D D2 R T sT sTR sTR2 cotT cT D2modDir nit
global FixInnerVor VorDirAtPole PsiDirFore PsiDirAft

[M,N] = size(psi);

BC = D2psi;
BC(1,:) = zeros(1,N);
BC(M,:) = 0.5*(R(M,:).*sT(M,:) ) .^2; 

if PsiDirAft,
%	BC(2:(M-1),1) = zeros(M-2,1);
%	BC(:,1) = zeros(M,1);
	BC(:,1) = 0.5*(R(:,1).*sT(:,1) ) .^2 .* (R(:,1) -1 )/(R(M,1) - 1);
end

if PsiDirFore,
%   BC(2:(M-1),N) = zeros(M-2,1);
	BC(:,N) = 0.5*(R(:,N).*sT(:,N) ) .^2 .* (R(:,N) -1 )/(R(M,N) - 1);
%   BC(:,N) = zeros(M,1);
end


psi = mldivide(D2modDir,BC);

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function PsiToU
global gr psi ur ut D2psi vor D D2 R T sT sTR sTR2 cotT cT

ur=   (D.dth*psi) ./sTR2;
ut= - (D.dr*psi) ./sTR;
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function PsiToVor
global gr psi ur ut D2psi vor D D2 R T sT sTR sTR2 cotT cT

vor  = ( ut + D.dr*ut - 1./sTR2*D.dth2*psi + ur.*cotT) ./ R;
return 
