function C = mldivide(T,B,par)
% mldivide - solve tensor-matrix equation for SparseTensors.
%
% Syntax: C = A\B
% or      C = mldivide(T,B,par)
% 
% The advantage of the second syntax is that additional parameters
% for the solution can be specifed; the first syntax uses default 
% parameters.
%
% par must be either an integer, or a structure with the following
% fields:
%
%  par.method :	integer specifying method for the corresponding
%						sparse matrix-vector equation. Try 'help sparfun' 
%						for info on these methods.
%						1: Gauss elemination (default)
%						2: bicg
%						3: bicgstab
%						4: cgs
%						5: gmres
%						6: qmr
%  par.tol :		Tolerance (applicable for method 2-6) (default 1e-2)
%  par.maxit :		Max. iteration (applicable for method 2-6) (default 1e3)
%  par.init :     Initial guess on the solution

if nargin<3,
   par = 1;
end

if ~isstruct(par)
	par.method = par;
	par.tol = 1e-2;
   par.maxit = 100;
   par.init = sparse(T.M,T.N);
end

if any([T.M T.N] ~= size(B))
   error('In A\B, if A is a SparseMatrix(M,N, ...), then B must be M-by-N')
end

par.init = reshape(par.init,T.M*T.N,1);

switch par.method
case 1
	C = reshape(full(T.mat \ reshape(B,T.M*T.N,1)),T.M,T.N);

case 2
	C = reshape(full(bicg(T.mat,reshape(B,T.M*T.N,1),par.tol,par.maxit,[],[],par.init)),T.M,T.N);
case 3
	C = reshape(full(bicgstab(T.mat,reshape(B,T.M*T.N,1),par.tol,par.maxit,[],[],par.init)),T.M,T.N);
case 4
	C = reshape(full(cgs(T.mat,reshape(B,T.M*T.N,1),par.tol,par.maxit,[],[],par.init)),T.M,T.N);
case 5
	C = reshape(full(gmres(T.mat,reshape(B,T.M*T.N,1),[],T.M*T.N,par.tol,par.maxit,[],[],par.init)),T.M,T.N);
case 6
	C = reshape(full(qmr(T.mat,reshape(B,T.M*T.N,1),par.tol,par.maxit,[],[],par.init)),T.M,T.N);
otherwise
   error('Unrecognized method')
end
