function TM=TensorMatrix(arg1,options)
% TM=TensorMatrix(ff,options)
%
% Constructs a TensorMatrix object, i.e. the terms in the equation which 
% corresponds to the advection-diffusion equation. Here ff is a flowfield 
% (see flowfield).
%
% Limitations: The discretisation scheme must be a method of the flowfield
% or of the underlying computational grid. 
%
% See also: flowfield, compgrid, SolveEquation.

if nargin<2
   options = [];
end

if isa(arg1,'flowfield'),
   TM = TMfromFlowField(arg1,options);
   return
end

error('Unrecognised arguments to the TensorMatrix constructor.')

function TM = TMfromFlowField(ff,options)

if isa(options,'double'),
   options.Peclet = options;
end

if ~isfield(options,'Method')
   options.Method = '2nd Order Central';
end

if strcmpi(options.Method,'2nd order central')
   D=SecondOrderCentral(ff);
   TM = BuildTM(D,ff,options);
   return   
end

if strcmpi(options.Method,'3-4 order upwind')
	D=FourthOrderUpwind(ff,options.Peclet);   
   TM = BuildTM(D,ff,options);
   return
end

if strcmpi(options.Method,'4th Order Central')
	D=FourthOrderCentral(ff);
   TM = BuildTM(D,ff,options);
   return   
end

% If we haven't returned so far ...
error('Unsupported method in TensorMatrix ...')

function TM = BuildTM(D,ff,options)

if ~isfield(options,'Peclet')
	options.Peclet = 1;
end

if isempty(options.Peclet)
	options.Peclet = 1;
end

if ~isfield(options,'Lambda')
	options.Lambda = 1;
end

if ~isfield(options,'BC')
   options.BC = 'Dirichlet';
end

[M,N] = size(D.dr);

A = SparseTensor(M,N);
B = sparse([],[],[],M,N,2*N);

% Extract grid positions and velocities from the flow field
R = RR(ff);
T = TT(ff);
ur = UR(ff);
ut = UT(ff);

Lambda = options.Lambda;
Peclet = options.Peclet;

% Laplace operator in spherical coor's
A = (2./R)*D.dr + D.dr2 + (cot(T)./R.^2)*D.dth + (1./R.^2)*D.dth2;

if Lambda<1,
   A = (Lambda + (1-Lambda)*R.^(4/3)) * A;
   
   % Add extra term due to turbulent diffusion
   A = A + (1-Lambda)*4/3*R.^(1/3)*D.dr;
end

% Subtract advection term
A = A + Peclet * ((-ur) * D.dr + (-ut./R)*D.dth);

% Take care of the boundary conditions at r=rmax.
A = SetUnity(A,M*ones(1,N),1:N);

% Take care of the boundary conditions at r=1.
if strcmpi(options.BC,'dirichlet')
   A = SetUnity(A,ones(1,N),1:N);
else
   for j=1:N,
      % Neumann type conditions
      A(1,j,:,:)= - D.dr(1,j,:,:);
   end
end

% Whether we have Dirichlet or Neumann, the right hand side of the BC at the 
% inner boundary is 1
B(1,:)=ones(1,N);

TM = struct('A',A,'B',B);
TM = class(TM,'TensorMatrix');

return
