function DiffOps=FourthOrderSkewed(cg)
%
% DiffOps=FourthOrderSkewed(cg)
%
% Returns discretized versions of d/dr, d/dtheta, d^2/dr^2 and d^2 /dtheta^2 
% corresponding to the logarithmic grid cg.
%
% Employs a third/fourth order skewed finite difference scheme.
%
% This function is for instance called from the constructor TensorMatrix
%
% See also: compgrid, FlowField, TensorMatrix

% Extract grid positions from the grid
R = RR(cg);  % Matrix of radii
T = TT(cg);  % Matrix of angles

% Mesh size in the computational grid
h = logrstep(cg);
k = psistep(cg);

% Number of radii and angles in the computational grid
[M,N] = size(cg);

% Find the corresponding rectangular grid.
rgr = rectgrid(M,N);

% Find the differential operators corresponding to this rectangular grid.
RecDiffOps = FourthOrderSkewed(rgr);

% Scale because our mesh size is not 1-by-1

DiffOps.drA = RecDiffOps.dxA * (1/ h);
DiffOps.drB = RecDiffOps.dxB * (1/ h);
DR2 = RecDiffOps.dx2 * (1/ h^2);

DiffOps.dthA = RecDiffOps.dyA * (1/ k);
DiffOps.dthB = RecDiffOps.dyB * (1/ k);
DTH2 = RecDiffOps.dy2 * (1/ k^2);

% Transfrom because the radius s logarithmically scaled.

DiffOps.dr2A = (1./R./R)*(DR2-DiffOps.drA);
DiffOps.dr2B = (1./R./R)*(DR2-DiffOps.drB);
DiffOps.drA =  (1./R)* DiffOps.drA;
DiffOps.drB =  (1./R)* DiffOps.drB;

% Transfrom because the angle is scaled.

DiffOps.dth2A = (1+GetGamma(cg)*cos(T)).^2 *DTH2 - GetGamma(cg)*sin(T)*DiffOps.dthA;
DiffOps.dth2B = (1+GetGamma(cg)*cos(T)).^2 *DTH2 - GetGamma(cg)*sin(T)*DiffOps.dthB;
DiffOps.dthA =  (1+GetGamma(cg)*cos(T))* DiffOps.dthA;
DiffOps.dthB =  (1+GetGamma(cg)*cos(T))* DiffOps.dthB;

return
