function gr=compgrid(Arg1,Arg2,Arg3,Arg4,Arg5)
%
% compgrid - Computational grid class; uniform in polar coordinates
%
% Constructor:
%
%		cg = compgrid;
%		cg = compgrid(options);
%		cg = compgrid(Rmax,M,N);
%		cg = compgrid(Rmax,M,N,name);
%		cg = compgrid(Rmax,M,N,name,gamma);
%
% When called as compgrid(Rmax,M,N), the three arguments
% must be scalar. A rectangular grid with 1< r < Rmax and 
% 0 < theta < pi is created. The mesh size is constant in the 
% angular direction and logarithmic in the radial direction. 
% M is the number of radii; N is the number of angles. 
% 
% When called with 5 arguments, gamma is a non-uniformity 
% parameter and should be in the range 0<gamma<1; 
% 0 corresponding to a uniform grid and 1 corresponding to a 
% singular grid.
%
% To see a list of methods, type "methods('compgrid')".
%

% Define defaults
Default = struct('Rmax',10,'M',20,'N',19,'Gamma',0/9,'Name','Computational grid');

% Verify inputs ...
if nargin==2 | nargin>5,
   error('Wrong number of arguments for compgrid. See ''help compgrid''.');
end

% With no arguements, return default grid.
if nargin==0,
   rmax=Default.Rmax;
   gamma = Default.Gamma;
   M=Default.M;
   N=Default.N;
   Name = Default.Name;
end

% With three arguments ...
if nargin==3,
   rmax = Arg1;
   M = Arg2;
   N = Arg3;
   Name = Default.Name;
   gamma = Default.Gamma;
end

% With four arguments ...
if nargin==4,
   rmax = Arg1;
   M = Arg2;
   N = Arg3;
   Name = Arg4;
   gamma = Default.Gamma;
end

% With five arguments ...
if nargin==5,
   rmax = Arg1;
   M = Arg2;
   N = Arg3;
   Name = Arg4;
   gamma = Arg5;
end

% One field is interpreted as a structure ...
if nargin==1,
   if ~isfield(Arg1,'Rmax'),
      Arg1.Rmax= Default.Rmax;
   end
   
   if ~isfield(Arg1,'M'),
      Arg1.M = Default.M;
   end
   
   if ~isfield(Arg1,'N'),
      Arg1.N = Default.N;
   end
   
   if ~isfield(Arg1,'Name'),
      Arg1.Name= Default.Name;
   end
   
   if ~isfield(Arg1,'Gamma'),
      Arg1.Gamma= Default.Gamma;
   end
   
   rmax = Arg1.Rmax;
   gamma = Arg1.Gamma;
   M = Arg1.M;
   N = Arg1.N;
   Name = Arg1.Name;
end

% Check that the input values are legal. If not, an error is thrown.

CheckScalar(gamma,[0,1]);
CheckScalar(rmax,[1,Inf]);
CheckScalar(N,[2,Inf],1);
CheckScalar(M,[2,Inf],1);

% Whew - OK, now input arguments are sorted and checked.
% Construct the grid.

R = logspace(0,log10(rmax),M);
R = R';

dpsi = pi/N;
T = FindT(N,gamma);

gr = struct('R',R,'T',T,'XX',[],'ZZ',[],'Gamma',gamma,'dpsi',dpsi,'Name',Name);

gr = class(gr,'compgrid');

return

function T = FindT(N,gamma)
% This is a Newton-Raphson iteration to solve for the scaled angles T 
% such that T + gamma*sin(T) = PSI
	dpsi = pi/N;
	PSIS = linspace(dpsi/2,pi-dpsi/2,N);

	T = PSIS;

	err = T + gamma*sin(T) - PSIS;
	i = 0;
	while max(abs(err))>1e-4,
      if i>100,
         error('Too many loops: Can''t seem to find the right scaled angles.');
      end
      
   	dq = 1 + gamma*cos(T);
	   T = T-err./dq;
		err = T + gamma*sin(T) - PSIS;
	end
return
