function ff=flowfield(psi,cg,Re,options)
%
% flowfield - consruct flowfield object corresponding to stream function
%
%   ff = flowfield(psi,gr,Re,options)
%
% returns the flowfield corresponding to the stream fcuntion psi at 
% Reynolds number Re, evaluated at the computational grid gr.
%
% The computation of the flow field is based on numeric differentiation, see
% 'help streamfct/diff'. The options (if present) are passed on to diff.
%
% An alternative syntax is 
%
%   ff = flowfield(ur,ut,gr,options)
%
% here ur and ut are matrices containing radial and tangential velocites.
% THey must have the same dimension as e.g. RR(cg).
% 
% See also: streamfct, compgrid, flowfield
%

%%%%%%%%%%%%%% Set default options whenever options are missing %%%%%%%%%%%

if nargin<1,
   psi = streamfct;
end

if nargin<2,
   cg = compgrid;
end

if nargin<3,
	Re = 1;
end

if nargin<4,
	options = [];   
end

% Check which type of constructor is needed: Initialise from 
% a streamfct or from numerical values?

if isa(psi,'streamfct'),
   %%%%%%%%%%%%%% Get velocities %%%%%%%%%%%
   [ur,ut] = PolarVel(psi,RR(cg),TT(cg),Re,options);
   vor = Vorticity(psi,cg,Re);
   name = getname(psi);
else
   %%%%% Numerical initialisation of the flow field %%%%%%%%%%%
   
   % Re-interpret input arguments
   ur = psi;
   ut = cg;
	cg = Re;   
   
   % Check that dimensions match
   if all([size(ur)==size(cg),size(ut)==size(cg)]),
      if isa(options,'char'),
         name = options;
      end
      if isfield(options,'name')
         name = options.name;
      end
      if ~exist('name'),
         name = 'Numerically assigned flow field';
      end
      
      if isfield(options,'vor'),
         vor = options.vor;
      else
         vor = [];
      end
   else
      error('Dimensions must match');
   end
end
%%%%%%%%%%%%%% Put output arguments in a structure %%%%%%%%%%%

ff = struct('Ur', ur, 'Ut', ut, 'Name', name,'vor',vor); 

ff = class(ff,'flowfield',cg);
