function DiffOps=FourthOrderCentral(cg,options)
% DiffOps=FourthOrderCentral(cg,options)
%
% Returns discretised versions of d/dX, d/dY, d^2/dX^2 and d^2 /dY^2 
% corresponding to the rectangular grid cg.
%
% Employs a fourth order central finite difference scheme.
%
% See also: compgrid, loggrid, rectgrid

if nargin<2,
	options = [];   
end

if isfield(options,'zeropole')
   ZeroPole = options.zeropole;
else
   ZeroPole = 0;
end

% Find number of points in the computational grid
M = GetM(cg);
N = GetN(cg);

% The following 4 tensors are discretisations of the differential 
% operators d/dY, d^2/dY^2, d/dr, d^2/dr^2 .
%
% They work the following way: If C is an M-by-N matrix, representing
% a concentration field, then dC/dr is obtained as DR*C

DY = SparseTensor(M,N,M*N*4);
D2Y = SparseTensor(M,N,M*N*5);
DX = SparseTensor(M,N,M*N*4);
D2X = SparseTensor(M,N,M*N*5);

% Fill in the terms in DY. This is a fourth order scheme
% which involves left and right cells. However, at the boundaries
% left cell equals same cell.

% Inner points 

d2left	= ones(M,N) * (  1/12);
dleft		= ones(M,N) * (- 2/3 );
dcenter	= zeros(M,N);
dright	= ones(M,N) * (  2/3 );
d2right	= ones(M,N) * ( -1/12);

% Left boundary

if ZeroPole,
   % Use a scheme which utilises that the functional value is zero    
    d2left(:,1)		= ones(M,1) * 0;   
	dleft(:,1)		= ones(M,1) * 0;   
    dcenter(:,1)	= ones(M,1) * 72/18; % 63
    dright(:,1)		= ones(M,1) * 0/18;  % 1
	d2right(:,1)	= ones(M,1) * 0;   
else
    dcenter(:,1)= dleft(:,1);
    dleft(:,1)  =zeros(M,1);
    dright(:,1) = dright(:,1) + d2left(:,1);
    d2left(:,1) =zeros(M,1);
end

% Next to left boundary

dleft(:,2) = dleft(:,2) + d2left(:,2);
d2left(:,2) = zeros(M,1);

% Right boundary

if ZeroPole,
   % Use a scheme which utilises that the functional value is zero    
	d2left(:,N)		= ones(M,1) * 0;   
	dleft(:,N)		= ones(M,1) * (-1/18);   
    dcenter(:,N)	= ones(M,1) * (-63/18);
    dright(:,N)		= ones(M,1) * 0;
	d2right(:,N)	= ones(M,1) * 0;   
else
    dcenter(:,N)    = dright(:,N);
    dright(:,N)     = zeros(M,1);
    dleft(:,N)      = dleft(:,N) + d2right(:,N);
    d2right(:,N)    = zeros(M,1);
end

% Next to right boundary

dright(:,N-1) = dright(:,N-1) + d2right(:,N-1);
d2right(:,N-1) = zeros(M,1);

DY = adddiag(DY,d2left,	[0 -2]);
DY = adddiag(DY,dleft,	[0 -1]);
DY = adddiag(DY,dcenter,[0  0]);
DY = adddiag(DY,dright,	[0  1]);
DY = adddiag(DY,d2right,[0  2]);

% Fill in the terms in DX. This is a central fourth order scheme
% which involves up and down cells. However, at the boundaries
% we have lower order schemes.

% Inner points 
d4down 	= zeros(M,N);
d3down 	= zeros(M,N);
d2down	= ones(M,N) * ( 1/12);
ddown		= ones(M,N) * (-2/3 );
dcenter	= zeros(M,N);
dup		= ones(M,N) * ( 2/3 );
d2up		= ones(M,N) * (-1/12);
d3up		= zeros(M,N);
d4up		= zeros(M,N);

% Lower boundary 
d2down(1,:)		= zeros(1,N);
ddown(1,:)		= zeros(1,N);
dcenter(1,:) 	= -25/12 * ones(1,N);
dup(1,:)		=  48/12 * ones(1,N);
d2up(1,:)		= -36/12 * ones(1,N) ;
d3up(1,:)		=  16/12 * ones(1,N) ;
d4up(1,:)		= - 3/12 * ones(1,N);

% Next to lower boundary
d2down	(2,:)	= zeros(1,N);
ddown	(2,:)	= - 3/12 * ones(1,N);
dcenter	(2,:)	= -10/12 * ones(1,N);
dup		(2,:)	=  18/12 * ones(1,N) ;
d2up	(2,:)	= - 6/12 * ones(1,N) ;
d3up	(2,:)	=   1/12 * ones(1,N);

% Next to upper boundary
d3down	(M-1,:)	= - 1/12 * ones(1,N);
d2down	(M-1,:)	=   6/12 * ones(1,N);
ddown	(M-1,:)	= -18/12 * ones(1,N) ;
dcenter	(M-1,:)	=  10/12 * ones(1,N) ;
dup		(M-1,:)	=   3/12 * ones(1,N);
d2up	(M-1,:) = zeros(1,N);

% Upper boundary
d4down	(M,:)	=   3/12 * ones(1,N);
d3down	(M,:)	= -16/12 * ones(1,N);
d2down	(M,:)	=  36/12 * ones(1,N) ;
ddown 	(M,:)	= -48/12 * ones(1,N) ;
dcenter	(M,:)	=  25/12 * ones(1,N);
dup		(M,:) = zeros(1,N);
d2up	(M,:) = zeros(1,N);

DX = adddiag(DX,d4down,	[-4 0]);
DX = adddiag(DX,d3down,	[-3 0]);
DX = adddiag(DX,d2down,	[-2 0]);
DX = adddiag(DX,ddown,	[-1 0]);
DX = adddiag(DX,dcenter,[ 0 0]);
DX = adddiag(DX,dup,	[ 1 0]);
DX = adddiag(DX,d2up,	[ 2 0]);
DX = adddiag(DX,d3up,	[ 3 0]);
DX = adddiag(DX,d4up,	[ 4 0]);

% Fill in the terms in D2Y. This is a fourth order central scheme
% which involves left, center, and right cells. 

% Inner points

d2left	= ones(M,N) * ( -1/12  );
dleft	= ones(M,N) * (  4/3   );
dcenter	= ones(M,N) * ( -5/2   );
dright	= ones(M,N) * (  4/3   );
d2right	= ones(M,N) * ( -1/12  );

% Left boundary

if ZeroPole,
   % Use a scheme which utilises that the functional value is zero    
	d2left(:,1)		= ones(M,1) * 0;   
	dleft(:,1)		= ones(M,1) * 0;   
    dcenter(:,1)	= ones(M,1) * 3;
    dright(:,1)		= ones(M,1) * 5/9;
	d2right(:,1)	= ones(M,1) * 0;   
else
   dcenter(:,1) = dcenter(:,1) + dleft(:,1);
   dleft(:,1)=zeros(M,1);
   dright(:,1) = dright(:,1) + d2left(:,1);
   d2left(:,1)=zeros(M,1);
end

% Next to left boundary

dleft(:,2) = dleft(:,2) + d2left(:,2);
d2left(:,2) = zeros(M,1);

% Right boundary

if ZeroPole,
   % Use a scheme which utilises that the functional value is zero    
	d2left(:,N)		= ones(M,1) * 0;   
	dleft(:,N)		= ones(M,1) * (5/9);
    dcenter(:,N)	= ones(M,1) * 3;
    dright(:,N)		= ones(M,1) * 0;
	d2right(:,N)	= ones(M,1) * 0;   
else
   dleft(:,N) = dleft(:,N) + d2right(:,N);
   dcenter(:,N) = dcenter(:,N) + dright(:,N);
   dright(:,N)=zeros(M,1);
   d2right(:,N)=zeros(M,1);
end

% Next to right boundary

dright(:,N-1) = dright(:,N-1) + d2right(:,N-1);
d2right(:,N-1) = zeros(M,1);

D2Y = adddiag(D2Y,d2left,	[0 -2]);
D2Y = adddiag(D2Y,dleft,		[0 -1]);
D2Y = adddiag(D2Y,dcenter,	[0  0]);
D2Y = adddiag(D2Y,dright,	[0  1]);
D2Y = adddiag(D2Y,d2right,	[0  2]);

% Fill in the terms in D2X. This is a central fourth order scheme
% which involves up and down cells. However, at the boundaries the
% scheme is schewed

% Inner points

d4down 	= zeros(M,N);
d3down 	= zeros(M,N);
d2down	= ones(M,N) * ( -1/12  );
ddown		= ones(M,N) * (  4/3   );
dcenter	= ones(M,N) * ( -5/2   );
dup		= ones(M,N) * (  4/3   );
d2up		= ones(M,N) * ( -1/12  );
d3up		= zeros(M,N);
d4up		= zeros(M,N);

% Lower boundary 

d4down(1,:) 	= ones(1,N) * (   0/12  );
d3down(1,:) 	= ones(1,N) * (   0/12  );
d2down(1,:)		= ones(1,N) * (   0/12  );
ddown(1,:)		= ones(1,N) * (   0/12  );
dcenter(1,:)	= ones(1,N) * (  35/12  );
dup(1,:)		= ones(1,N) * (-104/12  );
d2up(1,:)		= ones(1,N) * ( 114/12  );
d3up(1,:)		= ones(1,N) * (- 56/12  );
d4up(1,:)		= ones(1,N) * (  11/12  );

% Next to lower boundary 

d4down(2,:) 	= ones(1,N) * (   0/12  );
d3down(2,:) 	= ones(1,N) * (   0/12  );
d2down(2,:)		= ones(1,N) * (   0/12  );
ddown(2,:)		= ones(1,N) * (  11/12  );
dcenter(2,:)	= ones(1,N) * (- 20/12  );
dup(2,:)		= ones(1,N) * (   6/12  );
d2up(2,:)		= ones(1,N) * (   4/12  );
d3up(2,:)		= ones(1,N) * (-  1/12  );
d4up(2,:)		= ones(1,N) * (   0/12  );

% Next to upper boundary 

d4down(M-1,:) 	= ones(1,N) * (   0/12  );
d3down(M-1,:) 	= ones(1,N) * ( - 1/12  );
d2down(M-1,:)	= ones(1,N) * (   4/12  );
ddown(M-1,:)	= ones(1,N) * (   6/12  );
dcenter(M-1,:)	= ones(1,N) * ( -20/12  );
dup(M-1,:)		= ones(1,N) * (  11/12  );
d2up(M-1,:)		= ones(1,N) * (   0/12  );
d3up(M-1,:)		= ones(1,N) * (   0/12  );
d4up(M-1,:)		= ones(1,N) * (   0/12  );

% Upper boundary 

d4down(M,:) 	= ones(1,N) * (  11/12  );
d3down(M,:) 	= ones(1,N) * (- 56/12  );
d2down(M,:)		= ones(1,N) * ( 114/12  );
ddown(M,:)		= ones(1,N) * (-104/12  );
dcenter(M,:)	= ones(1,N) * (  35/12  );
dup(M,:)		= ones(1,N) * (   0/12  );
d2up(M,:)		= ones(1,N) * (   0/12  );
d3up(M,:)		= ones(1,N) * (   0/12  );
d4up(M,:)		= ones(1,N) * (   0/12  );

D2X = adddiag(D2X,d4down,	[-4 0]);
D2X = adddiag(D2X,d3down,	[-3 0]);
D2X = adddiag(D2X,d2down,	[-2 0]);
D2X = adddiag(D2X,ddown,	[-1 0]);
D2X = adddiag(D2X,dcenter,	[ 0 0]);
D2X = adddiag(D2X,dup,		[ 1 0]);
D2X = adddiag(D2X,d2up,		[ 2 0]);
D2X = adddiag(D2X,d3up,		[ 3 0]);
D2X = adddiag(D2X,d4up,		[ 4 0]);

DiffOps = struct('dx',DX,'dx2',D2X,'dy',DY,'dy2',D2Y);

return
