function DiffOps=FourthOrderSkewed(cg)
%
% DiffOps=FourthOrderSkewed(cg)
%
% Returns discretised versions of d/dx, d/dy, d^2/dx^2 and d^2 /dy^2 
% corresponding to the rectangular computational grid cg.
%
% Employs a third/fourth order upwind finite difference scheme.
%
% This function will typically only be called from the constructor TensorMatrix
%
% See also: compgrid, FlowField, TensorMatrix

% Mesh size in the computational grid
[M,N] = size(cg);

% The following 6 tensors are discretisations of the differential 
% operators d/dx, d^2/dx^2, d/dy, d^2/dy^2 .
%

DYA = SparseTensor(M,N,M*N*4);  % d/dy Skewed to the right
DYB = SparseTensor(M,N,M*N*4);  % d/dy Skewed to the left
D2Y = SparseTensor(M,N,M*N*5);  % d^2/dy^2 Central
DXA = SparseTensor(M,N,M*N*4);  % d/dx Skewed up 
DXB = SparseTensor(M,N,M*N*4);  % d/dx Skewed down 
D2X = SparseTensor(M,N,M*N*5);  % d^2/dx^2 Central

% Fill in the terms in DYA. This is a third order scheme, skewed to the right.

% Inner points 

d2left	= ones(M,N) * 0;
dleft		= ones(M,N) * (- 1/3 );
dcenter	= ones(M,N) * (- 1/2 );
dright	= ones(M,N) * (  1   );
d2right	= ones(M,N) * ( -1/6 );

% Left boundary

dcenter(:,1) 	= dcenter(:,1) + dleft(:,1);
dleft(:,1)		= zeros(M,1);
dright(:,1)		= dright(:,1) + d2left(:,1);
d2left(:,1)		= zeros(M,1);

% Next to left boundary

dleft(:,2)		= dleft(:,2) + d2left(:,2);
d2left(:,2)		= zeros(M,1);

% Right boundary

dcenter(:,N) 	= dcenter(:,N) + dright(:,N);
dright(:,N)		= zeros(M,1);
dleft(:,N)		= dleft(:,N) + d2right(:,N);
d2right(:,N)	= zeros(M,1);

% Next to right boundary

dright(:,N-1)	= dright(:,N-1) + d2right(:,N-1);
d2right(:,N-1) = zeros(M,1);

DYA = adddiag(DYA,d2left,	[0 -2]);
DYA = adddiag(DYA,dleft,	[0 -1]);
DYA = adddiag(DYA,dcenter,	[0  0]);
DYA = adddiag(DYA,dright,	[0  1]);
DYA = adddiag(DYA,d2right,	[0  2]);

% Fill in the terms in DYB. This is a third order scheme skewed to the left.

% Inner points 

d2left	= ones(M,N) * (  1/6 );
dleft		= ones(M,N) * (- 1   );
dcenter	= ones(M,N) * (  1/2  );
dright	= ones(M,N) * (  1/3 );
d2right	= ones(M,N) * (  0   );

% Left boundary

dcenter(:,1)	= dcenter(:,1) + dleft(:,1);
dleft(:,1)		= zeros(M,1);
dright(:,1) 	= dright(:,1) + d2left(:,1);
d2left(:,1)		= zeros(M,1);

% Next to left boundary

dleft(:,2)		= dleft(:,2) + d2left(:,2);
d2left(:,2)		= zeros(M,1);

% Right boundary

dcenter(:,N)	= dcenter(:,N) + dright(:,N);
dright(:,N)		= zeros(M,1);
dleft(:,N)		= dleft(:,N) + d2right(:,N);
d2right(:,N)	= zeros(M,1);

% Next to right boundary

dright(:,N-1)	= dright(:,N-1) + d2right(:,N-1);
d2right(:,N-1) = zeros(M,1);

DYB = adddiag(DYB,d2left,	[0 -2]);
DYB = adddiag(DYB,dleft,	[0 -1]);
DYB = adddiag(DYB,dcenter,	[0  0]);
DYB = adddiag(DYB,dright,	[0  1]);
DYB = adddiag(DYB,d2right,	[0  2]);

% Fill in the terms in DXA. This is a third order scheme skewed up.

% Inner points 

d3down		= ones(M,N) * 0;
d2down		= ones(M,N) * 0;
ddown			= ones(M,N) * (- 1/3 );
dcenter		= ones(M,N) * (- 1/2 );
dup			= ones(M,N) * (  1   );
d2up			= ones(M,N) * ( -1/6 );
d3up			= ones(M,N) * 0;

% Upper boundary - third order scheme

d3down(M,:)	= ones(1,N) * ( -2/6);
d2down(M,:)	= ones(1,N) * (  9/6);
ddown(M,:)	= ones(1,N) * (-18/6);
dcenter(M,:)= ones(1,N) * ( 11/6);
dup(M,:)		= ones(1,N) * (  0  );
d2up(M,:)	= ones(1,N) * (  0  );
d3up(M,:)	= ones(1,N) * (  0  );

% Next to upper boundary - third order schemes

d3down(M-1,:)	= ones(1,N) * (  0  );
d2down(M-1,:)	= ones(1,N) * (  1/6);
ddown(M-1,:)	= ones(1,N) * (- 6/6);
dcenter(M-1,:)	= ones(1,N) * (  3/6);
dup(M-1,:)		= ones(1,N) * (  2/6);
d2up(M-1,:)		= ones(1,N) * (  0  );
d3up(M-1,:)		= ones(1,N) * (  0  );

% Lower boundary - third order scheme

d3down(1,:)		= ones(1,N) * (  0  );
d2down(1,:)		= ones(1,N) * (  0  );
ddown(1,:)		= ones(1,N) * (  0  );
dcenter(1,:)	= ones(1,N) * (-11/6);
dup(1,:)			= ones(1,N) * ( 18/6);
d2up(1,:)		= ones(1,N) * (- 9/6);
d3up(1,:)		= ones(1,N) * (  2/6);

DXA = adddiag(DXA,d3down,	[-3 0]);
DXA = adddiag(DXA,d2down,	[-2 0]);
DXA = adddiag(DXA,ddown,	[-1 0]);
DXA = adddiag(DXA,dcenter,	[ 0 0]);
DXA = adddiag(DXA,dup,		[ 1 0]);
DXA = adddiag(DXA,d2up,		[ 2 0]);
DXA = adddiag(DXA,d3up,		[ 3 0]);

% Fill in the terms in DXB. This is a third order scheme skewed down.

% Inner points 

d3down		= ones(M,N) * 0;
d2down		= ones(M,N) * (  1/6);
ddown			= ones(M,N) * ( -6/6);
dcenter		= ones(M,N) * (  3/6);
dup			= ones(M,N) * (  2/6);
d2up			= ones(M,N) * ( 0   );
d3up			= ones(M,N) * 0;

% Upper boundary - third order scheme

d3down(M,:)	= ones(1,N) * ( -2/6);
d2down(M,:)	= ones(1,N) * (  9/6);
ddown(M,:)	= ones(1,N) * (-18/6);
dcenter(M,:)= ones(1,N) * ( 11/6);
dup(M,:)		= ones(1,N) * (  0  );
d2up(M,:)	= ones(1,N) * (  0  );
d3up(M,:)	= ones(1,N) * (  0  );

% Next to lower boundary - third order schemes

d3down(2,:)		= ones(1,N) * (  0  );
d2down(2,:)		= ones(1,N) * (  0  );
ddown(2,:)		= ones(1,N) * ( -2/6);
dcenter(2,:)	= ones(1,N) * ( -3/6);
dup(2,:)			= ones(1,N) * (  6/6);
d2up(2,:)		= ones(1,N) * ( -1/6);
d3up(2,:)		= ones(1,N) * (  0  );

% Lower boundary - third order scheme

d3down(1,:)		= ones(1,N) * (  0  );
d2down(1,:)		= ones(1,N) * (  0  );
ddown(1,:)		= ones(1,N) * (  0  );
dcenter(1,:)	= ones(1,N) * (-11/6);
dup(1,:)			= ones(1,N) * ( 18/6);
d2up(1,:)		= ones(1,N) * (- 9/6);
d3up(1,:)		= ones(1,N) * (  2/6);

DXB = adddiag(DXB,d3down,	[-3 0]);
DXB = adddiag(DXB,d2down,	[-2 0]);
DXB = adddiag(DXB,ddown,	[-1 0]);
DXB = adddiag(DXB,dcenter,	[ 0 0]);
DXB = adddiag(DXB,dup,		[ 1 0]);
DXB = adddiag(DXB,d2up,		[ 2 0]);
DXB = adddiag(DXB,d3up,		[ 3 0]);


% Fill in the terms in D2Y. This is a fourth order central scheme
% which involves left, center, and right cells. 

% Inner points

d2left	= ones(M,N) * ( -1/12 );
dleft		= ones(M,N) * (  4/3  );
dcenter	= ones(M,N) * ( -5/2  );
dright	= ones(M,N) * (  4/3  );
d2right	= ones(M,N) * ( -1/12 );

% Left boundary

dcenter(:,1) = dcenter(:,1) + dleft(:,1);
dleft(:,1)=zeros(M,1);
dright(:,1) = dright(:,1) + d2left(:,1);
d2left(:,1)=zeros(M,1);

% Next to left boundary

dleft(:,2) = dleft(:,2) + d2left(:,2);
d2left(:,2) = zeros(M,1);

% Right boundary

dcenter(:,N) = dcenter(:,N) + dright(:,N);
dright(:,N)=zeros(M,1);
dleft(:,N) = dleft(:,N) + d2right(:,N);
d2right(:,N)=zeros(M,1);

% Next to right boundary

dright(:,N-1) = dright(:,N-1) + d2right(:,N-1);
d2right(:,N-1) = zeros(M,1);

D2Y = adddiag(D2Y,d2left,	[0 -2]);
D2Y = adddiag(D2Y,dleft,	[0 -1]);
D2Y = adddiag(D2Y,dcenter,	[0  0]);
D2Y = adddiag(D2Y,dright,	[0  1]);
D2Y = adddiag(D2Y,d2right,	[0  2]);

% Fill in the terms in D2X. This is a central fourth order scheme
% which involves up and down cells. However, at the boundaries the
% scheme is schewed

% Inner points

d4down 	= zeros(M,N);
d3down 	= zeros(M,N);
d2down	= ones(M,N) * ( -1/12  );
ddown		= ones(M,N) * ( 16/12   );
dcenter	= ones(M,N) * (-30/12   );
dup		= ones(M,N) * ( 16/12   );
d2up		= ones(M,N) * ( -1/12  );
d3up		= zeros(M,N);
d4up		= zeros(M,N);

% Lower boundary 

d4down(1,:) 	= ones(1,N) * (   0/12  );
d3down(1,:) 	= ones(1,N) * (   0/12  );
d2down(1,:)		= ones(1,N) * (   0/12  );
ddown(1,:)		= ones(1,N) * (   0/12  );
dcenter(1,:)	= ones(1,N) * (  35/12  );
dup(1,:)			= ones(1,N) * (-104/12  );
d2up(1,:)		= ones(1,N) * ( 114/12  );
d3up(1,:)		= ones(1,N) * (- 56/12  );
d4up(1,:)		= ones(1,N) * (  11/12  );

% Next to lower boundary 

d4down(2,:) 	= ones(1,N) * (   0/12  );
d3down(2,:) 	= ones(1,N) * (   0/12  );
d2down(2,:)		= ones(1,N) * (   0/12  );
ddown(2,:)		= ones(1,N) * (  11/12  );
dcenter(2,:)	= ones(1,N) * (- 20/12  );
dup(2,:)			= ones(1,N) * (   6/12  );
d2up(2,:)		= ones(1,N) * (   4/12  );
d3up(2,:)		= ones(1,N) * (-  1/12  );
d4up(2,:)		= ones(1,N) * (   0/12  );

% Next to upper boundary 

d4down(M-1,:) 	= ones(1,N) * (   0/12  );
d3down(M-1,:) 	= ones(1,N) * ( - 1/12  );
d2down(M-1,:)	= ones(1,N) * (   4/12  );
ddown(M-1,:)	= ones(1,N) * (   6/12  );
dcenter(M-1,:)	= ones(1,N) * ( -20/12  );
dup(M-1,:)		= ones(1,N) * (  11/12  );
d2up(M-1,:)		= ones(1,N) * (   0/12  );
d3up(M-1,:)		= ones(1,N) * (   0/12  );
d4up(M-1,:)		= ones(1,N) * (   0/12  );

% Upper boundary 

d4down(M,:) 	= ones(1,N) * (  11/12  );
d3down(M,:) 	= ones(1,N) * (- 56/12  );
d2down(M,:)		= ones(1,N) * ( 114/12  );
ddown(M,:)		= ones(1,N) * (-104/12  );
dcenter(M,:)	= ones(1,N) * (  35/12  );
dup(M,:)			= ones(1,N) * (   0/12  );
d2up(M,:)		= ones(1,N) * (   0/12  );
d3up(M,:)		= ones(1,N) * (   0/12  );
d4up(M,:)		= ones(1,N) * (   0/12  );

D2X = adddiag(D2X,d4down,	[-4 0]);
D2X = adddiag(D2X,d3down,	[-3 0]);
D2X = adddiag(D2X,d2down,	[-2 0]);
D2X = adddiag(D2X,ddown,	[-1 0]);
D2X = adddiag(D2X,dcenter,	[ 0 0]);
D2X = adddiag(D2X,dup,		[ 1 0]);
D2X = adddiag(D2X,d2up,		[ 2 0]);
D2X = adddiag(D2X,d3up,		[ 3 0]);
D2X = adddiag(D2X,d4up,		[ 4 0]);

DiffOps = struct('dxA',DXA,'dxB',DXB,'dx2',D2X, ...
   'dyA',DYA,'dyB',DYB,'dy2',D2Y);

return
