function DiffOps=SecondOrderCentral(cg,symm)
%
% DiffOps=SecondOrderCentral(cg)
%
% or 
%
% DiffOps=SecondOrderCentral(cg,symm)
%
% symm should be 1 (symmetric field) or -1 (schew symmetric field).
% If symm is absent it will be taken to be 1.
%
% Returns discretised versions of d/dx, d/dy, d^2/dx^2 and d^2 d/dy^2 
% corresponding to the rectangular grid cg.
%
% Employs a second order central finite difference scheme.
%
% This function will typically only be called from the constructor TensorMatrix
%
% See also: rectgrid, compgrid, FlowField, TensorMatrix

if nargin<2,
   symm = 1;
end

M = GetM(cg);
N = GetN(cg);

% The following 4 tensors are discretisations of the differential 
% operators d/dY, d^2/dY^2, d/dr, d^2/dr^2 .
%
% They work the following way: If C is an M-by-N matrix, representing
% a concentration field, then dC/dr is obtained as DR*C

DY = SparseTensor(M,N,M*N*2);
D2Y = SparseTensor(M,N,M*N*3);
DX = SparseTensor(M,N,M*N*2);
D2X = SparseTensor(M,N,M*N*3);

% Fill in the terms in DY. This is a central second order scheme
% which involves left and right cells. However, at the boundaries
% left cell equals same cell.

dcenter = zeros(M,N);
dcenter(:,1) = -symm*ones(M,1)/2;
dcenter(:,N) = symm*ones(M,1)/2;
dright = ones(M,N)/2;
dleft = -ones(M,N)/2;
dright(:,N) = zeros(M,1);
dleft(:,1) = zeros(M,1);

DY = adddiag(DY,dcenter,[0 0]);
DY = adddiag(DY,dleft,[0 -1]);
DY = adddiag(DY,dright,[0 1]);

% Fill in the terms in DX. This is a central second order scheme
% which involves up and down cells. However, at the boundaries
% we have a first order scheme in stead.

dcenter = zeros(M,N);
dup = ones(M,N)/2;
ddown = -ones(M,N)/2;

dcenter(1,:) = -ones(1,N);
dup(1,:) = ones(1,N);
ddown(1,:) = zeros(1,N);

ddown(M,:) = -ones(1,N);
dcenter(M,:) = ones(1,N);
dup(M,:) = zeros(1,N);

DX = adddiag(DX,dcenter,[0 0]);
DX = adddiag(DX,dup,[1 0]);
DX = adddiag(DX,ddown,[-1 0]);

% Fill in the terms in D2Y. This is a central second order scheme
% which involves left, center, and right cells. However, at the boundaries
% left cell equals center cell.

dcenter = -2*ones(M,N);
dcenter(:,1) = dcenter(:,1) + symm*ones(M,1);
dcenter(:,N) = dcenter(:,N) + symm*ones(M,1);
dright = ones(M,N);
dleft =  ones(M,N);
dright(:,N) = zeros(M,1);
dleft(:,1) = zeros(M,1);

D2Y = adddiag(D2Y,dcenter,[0 0]);
D2Y = adddiag(D2Y,dleft,[0 -1]);
D2Y = adddiag(D2Y,dright,[0 1]);

% Fill in the terms in D2X. This is a central second order scheme
% which involves up and down cells. At the boundaries the scheme is
% skewed inwards.

d2up = zeros(M,N);
dup = ones(M,N);
dcenter = -2*ones(M,N);
ddown = ones(M,N);
d2down = zeros(M,N);

dup(M,:) = zeros(1,N);
dcenter(M,:) = ones(1,N);
ddown(M,:) = -2*ones(1,N);
d2down(M,:) = ones(1,N);

d2up(1,:) = ones(1,N);
dup(1,:) = -2*ones(1,N);
dcenter(1,:) = ones(1,N);
ddown(1,:) = zeros(1,N);

D2X = adddiag(D2X,d2up,[2 0]);
D2X = adddiag(D2X,dup,[1 0]);
D2X = adddiag(D2X,dcenter,[0 0]);
D2X = adddiag(D2X,ddown,[-1 0]);
D2X = adddiag(D2X,d2down,[-2 0]);

DiffOps = struct('dx',DX,'dx2',D2X,'dy',DY,'dy2',D2Y);

return
