function dpsi=Diff(psi,r,t,Re,options)
%
% Diff - numerical differentiation of streamfunction.
%
%   dpsi=Diff(psi,r,t,Re,options)
% 
% Computes a numerical approximation to the dpsi/dr and dpsi/dt at points
% r,t for Reynolds number Re.
%
% Default options are
%
%    options.order = 4
%    options.dr    = 0.001
%    options.dt    = 0.001
%
% Order can be 2 or 4. The stepsizes dr or dt can be scalars or arrays 
% corresponding to r.

if nargin<5,
   options.opt = 0;
end

if ~isfield(options,'order')
   options.order = 2;
end

if ~isfield(options,'dt')
   options.dt = 0.001;
end

if ~isfield(options,'dr')
   options.dr = 0.001;
end

% Here we make sure that (TT + dt) and TT differ by an exactly
% representable number. C.f. 'Numerical recipes in C', p. 186

temp =  t+ options.dt;
donothing(temp);
dt = temp - t;

temp = r + options.dr;
donothing(temp);
dr = temp - r;

% Do the math

if options.order==2,
   dpsi.dt = (eval(psi,r,t + dt,Re) - eval(psi,r,t-dt,Re)) / 2 ./ dt;
	dpsi.dr = (eval(psi,r + dr,t,Re) - eval(psi,r-dr,t,Re )) / 2 ./ dr;
	return   
end

if options.order==4,
   dpsi.dt = ( eval(psi,r,t - 2* dt	,Re)/12	- eval(psi,r,t-dt			,Re)*2/3 ...
   		   +	eval(psi,r,t + dt		,Re)*2/3 - eval(psi,r,t - 2* dt	,Re)/12 ) ./ dt;
   
   dpsi.dr = ( eval(psi,r - 2*dr	,t	,Re)/12	- eval(psi,r - dr		,t ,Re)*2/3 ...
      		+	eval(psi,r + dr	,t	,Re)*2/3 - eval(psi,r - 2*dr	,t ,Re)/12 ) ./ dr;
   return
end

% If we haven't returned yet ...

error('Unrecognized order in Diff')

function donothing(temp)
	temp=temp;
return

