function psi=streamfct(str,name)
% streamfct - Streamfunction class constructor
%
% Syntax: psi = streamfct(str,name)
%
% Here, str is an inline expression in r,theta, and Re,
% while name is - whoa - the name of the resulting 
% streamfunction.
%
% Try e.g. psi = streamfct('0.5*r.^2^*(sin(theta).^2)','Free flow')
%
% Alternatively, you may specify a built-in stream function:
%
%    psi = streamfct('stokes')
%
% generates - whoa again - the Stokes' stream function. Other 
% built-in functions are the Oseen solution 'Oseen' as well as
% the Proudman-Pearsson family 'PP Multiplicative', 'PP Additive',
% 'PP Inner', 'PP Outer', 'PP I/O'. Final choices are 'Pump', 
% 'Free', and 'PP Convex'.


if nargin<2,
   name = 'Unnamed stream function';
end

if nargin==0,
   str = 'stokes';
end

if strcmpi(str,'pump')
   str='0.25*sin(theta).^2.*(-3*r+ 1./r)';
   name='Spherical pump (Stokes)';
end

if strcmpi(str,'free')
   str='0.5*(r.^2).* (sin(theta).^2)';
   name = 'Free stream function';
end

if strcmpi(str,'stokes')
   str='0.25*(2 * r.^2 + 1./r - 3* r).* (sin(theta)).^2';
   name = 'Stokes stream function';
end

if strcmpi(str,'oseen')
   str = strcat('1/4*(2*r.^2 + 1./r) .* sin (theta).^2 - 1.5./r.*(1+cos(theta))',...
       '.* ( 1 - exp(-0.5*Re*r.*(1-cos(theta))))');
   name = 'Oseen';
end

if strcmpi(str(1:2),'pp')
   % Inner stream function.
   % Call as psi_i(r,theta,Re)
   ppistring = strcat( ...
      '0.25*(r-1).^2 .*(sin(theta).^2) .* (', ...
      '(1 + 3/8*Re)*(2+1./r) - ', ...
      '3/8 * Re * (2 + 1./r .* (1 + 1./r ))', ...
      '.* cos(theta) ) ');
   
   % Outer stream function 
   % Call as psi_o(r,theta,Re)
   ppostring =  strcat( ...
      '0.5*r.^2.*(sin(theta).^2)', ...
      '- 1.5/Re * (1+cos(theta)).* ', ...
      '(1 - exp(-0.5*Re*r.*(1-cos(theta))))');
   
   % Inner expansion of out expansion - [Acheson, 1990, p. 228]
   % Call as psi_io(r,theta,Re)
   ppiostring =  strcat( ...
      '0.25*r.^2 .* (sin(theta)).^2 .*', ...
      '(2 + 0.75 * Re * ( 1 - cos(theta)) - 3./ r)') ;
   
   % Convex combination of inner and outer expansion (Thygesen :-)
   lambdastr = '(1./(1+(r-1).^2))';
   ppconstring = strcat( ...
      lambdastr , '.* ( ',ppistring, ... 
      ') + (1 - ',lambdastr,') .* (',ppostring,')');
   
   % Additve composite expansion - [van Dyke, 1964, p. ...]
   % Call as psi_ca(r,theta,Re)
   
   psi_castring = strcat('(',ppistring,')+(',ppostring,')-(',ppiostring,')');
   
   % Multiplicative composite expansion - [van Dyke, 1964, ...]
   % Call as psi_cm(r,theta,Re)
   
   psi_cmstring = strcat('(',ppistring,').*(',ppostring,')./(',ppiostring,')');
   
   switch lower(str(1:6))
   case 'pp mul'
      str = psi_cmstring;
      name = 'Proudman-Pearson Multiplicative';
   case 'pp add'
      str = psi_castring;
      name = 'Proudman-Pearson Additive';
   case 'pp inn'
      str = ppistring;
      name = 'Proudman-Pearson Inner';
   case 'pp con'
      str = ppconstring;
      name = 'Proudman-Pearson Convex';
   case 'pp out'
      str = ppostring;
      name = 'Proudman-Pearson Outer';
   case 'pp i/o'
      str = ppiostring;
      name = 'Proudman-Pearson Inner/Outer';
   otherwise
      error('Unrecognized name of string function')
   end
end

v=scalarfield(name);

psi.fct=inline(str,'r','theta','Re');

psi=class(psi,'streamfct',v);

